import knowledge_repo as kr
import nbformat
from os.path import join, exists
from knowledge_repo.app.deploy import KnowledgeDeployer, get_app_builder

default_header = '---\ntitle: This is a Knowledge Template Header\nauthors:\n- sally_smarts \n- wesley_wisdom\ntags:\n- knowledge\n- example\ncreated_at: 2016-06-29\nupdated_at: 2016-06-30\ntldr: This is short description of the content and findings of the post.\n---'


def repository(name):
    if exists(join('knowledgerepos', name)):
        return kr.KnowledgeRepository.for_uri(name)
    return kr.KnowledgeRepository.create_for_uri(name)


def prep(file, out):
    nb = nbformat.read(file)
    if 'kr' in nb.cells[0].metadata:
        return
    kr_node = nbformat.notebooknode.NotebookNode()
    kr_node.metadata = nbformat.notebooknode.NotebookNode()
    kr_node.metadata['kr'] = True
    kr_node.cell_type = 'raw'
    kr_node.source = default_header
    nb.cells.insert(0, kr_node)
    with open(out, 'w') as fp:
        nbformat.write(nb, fp)


def post(repo, name, message="New revision"):
    kp = kr.KnowledgePost.from_file(name, format='ipynb')
    if repo.has_post(name):
        repo.add(kp, path=name, update=True, message=message)
        repo.submit(name)
    else:
        repo.add(kp, path=name, message=message)
        repo.submit(name)


def deploy(repos, config=None):
    app_builder = get_app_builder(repos,
                                  debug=True,
                                  db_uri=None,
                                  config=config,
                                  INDEXING_ENABLED=True)
    KnowledgeDeployer.using('gunicorn')(
        app_builder,
        host='0.0.0.0',
        port=8890
        ).run()



from notebook.utils import url_path_join
from notebook.base.handlers import IPythonHandler


class SubmitKnowledgeHandler(IPythonHandler):
    def get(self):
        argument = self.get_argument('notebook')
        self.finish(argument)


def load_jupyter_server_extension(nb_server_app):
    """
    Called when the extension is loaded.

    Args:
        nb_server_app (NotebookWebApplication): handle to the Notebook webserver instance.
    """
    web_app = nb_server_app.web_app
    host_pattern = '.*$'
    route_pattern = url_path_join(web_app.settings['base_url'], '/kr/submit')
    print('Installing knowledgerepo handler on path %s', route_pattern)
    web_app.add_handlers(host_pattern, [(route_pattern, SubmitKnowledgeHandler)])



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='knowledgelab',
    version='0.0.1',
    description='Knowledge Repo integration for JupyterLab',
    long_description=long_description,
    url='https://github.com/timkpaine/knowledgelab',
    download_url='https://github.com/timkpaine/knowledgelab/archive/v0.0.1.tar.gz',
    author='Tim Paine',
    author_email='t.paine154@gmail.com',
    license='LGPL',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='analytics jupyter',

    packages=find_packages(exclude=['tests',]),
    zip_safe=False,

    # entry_points={
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
)
